import datetime

from rich.table import Table
from rich.text import Text
from rich.live import Live
from rich import box, print as rich_print

COLOR_GREEN = "light_green"
COLOR_GRAY = "gray70"
NEWLINE = "\n"

def format_day(date): 
    _, _, day = date.split('-')
    return day

def format_to_date(date_str):
    datum, hours = date_str.split('T')
    year, month, day = datum.split('-')
    hours_template, _ = hours.split('-')
    hour, minute, second = hours_template.split('.')[0].split(':')
    date = datetime.datetime(int(year), int(month), int(day), int(hour), int(minute), int(second))
    return date

def format_to_hours(date_str):
    _, hours = date_str.split('T')
    hours_template, _ = hours.split('-')
    hour, minute, seconds = hours_template.split('.')[0].split(':')
    return f"{hour}:{minute}:{seconds}"

def trunc_numbers(number): 
    return "R$ " + str("{:.2f}".format(number))

def match_project(project_id, projects): 
    return next((project for project in projects if str(project["id"]) == str(project_id)), None)

def printTasks(tasks_tuple, projects): 
    table = Table(
        title="Month Tasks",
        box=box.DOUBLE_EDGE,
        title_style="white bold",
        caption_style=COLOR_GRAY,
        border_style=COLOR_GRAY,
    )

    table.add_column("Day", style=COLOR_GREEN)
    table.add_column("Project", style=COLOR_GREEN)
    table.add_column("Description", style=COLOR_GRAY)
    table.add_column("Start", style=COLOR_GRAY)
    table.add_column("End", style=COLOR_GRAY)
    table.add_column("Duration")
    table.add_column("Cost (R$)")
    
    wage = 0

    for date, tasks in tasks_tuple:
        for task in tasks["tasks"]: 
            start = task['start']
            end = task['end']
            duration = format_to_date(end) - format_to_date(start)
            project = match_project(task['project_id'], projects)
            table.add_row(
                format_day(date),
                f"[{project['tag_color']}]{project['name']}",
                task['description'],
                format_to_hours(task['start']),
                format_to_hours(task['end']),
                str(duration),
                f"[light_green]{trunc_numbers(task['cost'])}"
            )
            wage = wage + task['cost']
    table.add_row(
        "",
        "",
        "",
        "",
        "",
        "[green]Total:",
        f"[green]{trunc_numbers(wage)}"
    )
    return table

def convert_seconds_to_hours(seconds): 
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    return "%02i:%02i:%02i" % (h, m, s)

def has_sent_nfe(invoice): 
    if invoice['id'] == None:
        return "[red]:heavy_check_mark:"
    return f"[{COLOR_GREEN}]:heavy_check_mark:"

def print_reports(reports): 
    table = Table(
        title="Reports",
        box=box.DOUBLE_EDGE,
        title_style="white bold",
        caption_style=COLOR_GRAY,
        border_style=COLOR_GRAY,
    )

    table.add_column("Month", style=COLOR_GREEN)
    table.add_column("Total Hours", style=COLOR_GRAY)
    table.add_column("Hour Value", style=COLOR_GRAY)
    table.add_column("Sub Total", style=COLOR_GRAY)
    table.add_column("Discount", style=COLOR_GRAY)
    table.add_column("Final Value", style=COLOR_GRAY)
    table.add_column("NF-e")

    current_month = len(reports)
    for report in reports:
        sub_total = report['duration'] / 1000 / 60 / 60 * report['hour_value']
        table.add_row(
            f"{current_month}",
            convert_seconds_to_hours(report['duration'] / 1000),
            trunc_numbers(report['hour_value']),
            trunc_numbers(sub_total),
            trunc_numbers(report['current_discount']),
            trunc_numbers(sub_total - report['current_discount']),
            has_sent_nfe(report['invoice'])

        )
        current_month = current_month - 1

    return table


import calendar

months = {
    "Jan": 1,
    "Fev": 2,
    "Mar": 3,
    "Abr": 4,
    "Mai": 5,
    "Jun": 6,
    "Jul": 7,
    "Ago": 8,
    "Set": 9,
    "Out": 10,
    "Nov": 11,
    "Dez": 12,
}

def get_last_day_of_month(month_number, year): 
    _, last_day = calendar.monthrange(int(year), int(month_number))
    return last_day


import json

def write(json_file): 
    with open('config.json', 'w') as file:
        json.dump(json_file, file)

def load():
    with open('config.json', 'r') as file:
        return json.load(file)


import click
from rich import print
from datetime import datetime

from api import _sign_in, _sign_out, _tasks, _reports
from config import write, load
from sensitive_data import email, password
from beautify import printTasks, print_reports

@click.group()
def cli():
    ...


@cli.command(help="Login to Labor")
def sign_in():
    email = click.prompt('Enter your email', type=str)
    password = click.prompt('Enter your password', type=str, hide_input=True)
    data, status, headers = _sign_in(email, password)

    saved_headers = {
        'access-token': headers['access-token'],
        'client': headers['client'],
        'expiry': headers['expiry'],
        'uid': headers['uid'],
        'token-type': headers['token-type']
    }

    config_json = {
        'saved_headers': saved_headers,
        'data': data,
    }

    if status == 200: 
        write(config_json)
        print("User logged in successfully")
    else: print("Error logging in")

@cli.command(help="Logout from Labor")
def sign_out():
    logged_user = load()
    headers = logged_user['saved_headers']
    status = _sign_out(headers)
    if status == 200:
        write({})
    else: 
        print('Error while signing out')


@cli.command(help="Default is current month, use month and year in numbers")
@click.argument("month", required=False)
@click.argument("year", required=False)
def tasks(month, year):
    try:
        if not year:
            year = datetime.now().year
        if not month: 
            month = datetime.now().month
        logged_user = load()
        data, status, projects = _tasks(logged_user, month, year)
        if status == 200: 
            print(printTasks(dict.items(data), projects))
        else: 
            print(f"Error while retrieving data: {status}")
    except: 
        print("You need to login first")

@cli.command(help="Get reports")
@click.argument("year", required=False)
def reports(year):
    try: 
        if not year:
            year = datetime.now().year
        logged_user = load()
        data, status = _reports(logged_user, year)
        if status == 200:
            print(print_reports(data))
    except:
        print("You need to login first")

if __name__ == "__main__":
    cli()



import httpx
import json
from config import write
from date_utils import get_last_day_of_month

BASE_URL = "https://api.getlabor.com.br/"

def update_header_interceptor(logged_user, headers):
    if not headers['access-token']: return
    logged_user["saved_headers"] = {
        'access-token': headers['access-token'],
        'client': headers['client'],
        'expiry': headers['expiry'],
        'uid': headers['uid'],
        'token-type': headers['token-type']
    }
    write(logged_user)

def _request(url, logged_user):
    headers = logged_user["saved_headers"]
    res = httpx.get(url, headers=headers)
    update_header_interceptor(logged_user, res.headers)
    return res

def _sign_in(email, password):
    url = BASE_URL + "auth/sign_in"
    res = httpx.post(url, json={"email": email, "password": password})
    return json.loads(res.text)['data'], res.status_code, res.headers

def _sign_out(headers):
    url = BASE_URL + "auth/sign_out"
    res = httpx.delete(url, headers=headers)
    return res.status_code

def _tasks(logged_user, month, year):
    user_id = logged_user["data"]["id"]
    last_day = get_last_day_of_month(month, year)
    url = f"https://api.getlabor.com.br/tasks?cost=true&ending={year}-{month}-{last_day}T23:59:59.543Z&includes%5B%5D=planned_task&starting={year}-{month}-01T00:00:00.000Z&user_id%5B%5D={user_id}"
    projects = _request(BASE_URL + 'projects', logged_user)
    res = _request(url, logged_user)
    return json.loads(res.text), res.status_code, json.loads(projects.text)

def _reports(logged_user, year):
    res = _request(BASE_URL + f'reports?&year={year}', logged_user)
    return json.loads(res.text), res.status_code